/*
 * Project: irond
 * Package: src.de.fhhannover.inform.iron.mapserver.binding
 * File:    SimpleValidationHandler.java
 *
 * Copyright (C) 2010-2011 Fachhochschule Hannover
 * Ricklinger Stadtweg 118, 30459 Hannover, Germany 
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package de.fhhannover.inform.iron.mapserver.binding;

import javax.xml.bind.ValidationEvent;
import javax.xml.bind.ValidationEventHandler;

/**
 * A simple validation handler which aborts the unmarshalling process
 * if an error or fatal error occurs. The corresponding message is
 * can be queried later using the {@link #getErrorMessage()} method.
 * 
 * @author aw
 *
 */
class SimpleValidationHandler implements ValidationEventHandler {
	
	private boolean mErrorOccured;
	private String mErrorMessage;
	
	public SimpleValidationHandler() {
		mErrorMessage = null;
		mErrorOccured = false;
	}
	

	@Override
	public boolean handleEvent(ValidationEvent event) {
		
		if (event.getSeverity() == ValidationEvent.ERROR ||
				event.getSeverity() == ValidationEvent.FATAL_ERROR) {
			mErrorMessage = event.getMessage();
			mErrorOccured = true;
		}
		return !mErrorOccured;
	}


	public String getErrorMessage() {
		return mErrorMessage;
	}


	public boolean hasErrorOccured() {
		return mErrorOccured;
	}
}
